package com.example.drools.testing;

import java.util.ArrayList;
import java.util.List;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;

public class Main {

	public static void main(String[] args) {
		final KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
				.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("ShoppingRules.drl",Main.class),
				ResourceType.DRL);
		if (kbuilder.hasErrors()) {
			System.out.print("errors:\n" + kbuilder.getErrors().toString());
			return;
		}else{
			System.out.println("No errors in kbuilder");
		}

		final KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		final StatefulKnowledgeSession ksession = kbase
				.newStatefulKnowledgeSession();

		RuleMessage m = new RuleMessage();
		ksession.insert(m);
		
		List<CartItem> items = new ArrayList<Main.CartItem>();
		
		items.add(new CartItem(1,"A",100D));
		items.add(new CartItem(2,"A",100D));
		items.add(new CartItem(3,"B",100D));
//		items.add(new CartItem(4,"A",100D));
		
		System.out.println("Initial Items");
		printItems(items);
		
		for(CartItem ci : items){
			ksession.insert(ci);
		}
		
		ksession.setGlobal("bundler", new Bundler());
		
		ksession.fireAllRules();
		System.out.println("Message:"+m.getMessage());
		
		System.out.println("end list");
		printItems(items);

		System.out.println("Finished executings");

	}
	
	public static void printItems(List<CartItem> items){
		System.out.println("----Start----");
		for(CartItem ci : items){
			System.out.println(ci.toString());
		}
		System.out.println("----end----");
	}

	
	public static class ShoppingCart{

	}
	
	public static class RuleMessage{
		private String message = "";

		public String getMessage() {
			return message;
		}

		public void setMessage(String message) {
			this.message = message;
		}
		public void addMessage(String msg){
			message += ("\n" + msg);
		}
	}
	
	public static class CartItem{
		private Integer id;
		private String productId;
		private Double price;
		private Integer bundleId = 0;
		private Integer aggregatorId = 0;
		private boolean processed = false;
		private String promoId;
		private Double promoItemDiscount = 0D;
		
		public CartItem() {
		}
		
		public CartItem(Integer id, String productId, Double price){
			this.id=id;
			this.productId = productId;
			this.price = price;
		}
		
		
		public Integer getId() {
			return id;
		}
		public void setId(Integer id) {
			this.id = id;
		}
		public String getProductId() {
			return productId;
		}
		public void setProductId(String productId) {
			this.productId = productId;
		}
		public Double getPrice() {
			return price;
		}
		public void setPrice(Double price) {
			this.price = price;
		}
		public Integer getBundleId() {
			return bundleId;
		}
		public void setBundleId(Integer bundleId) {
			this.bundleId = bundleId;
		}
		public Integer getAggregatorId() {
			return aggregatorId;
		}
		public void setAggregatorId(Integer aggregatorId) {
			this.aggregatorId = aggregatorId;
		}
		
		
		@Override
		public String toString() {
			return "id["+id+"] pid["+productId+"] processed["+processed+"] bid["+bundleId+"] aid["+aggregatorId+"] price["+price+"]";
		}

		public Double getPromoItemDiscount() {
			return promoItemDiscount;
		}

		public void setPromoItemDiscount(Double promoItemDiscount) {
			this.promoItemDiscount = promoItemDiscount;
		}

		public boolean isProcessed() {
			return processed;
		}

		public void setProcessed(boolean processed) {
			this.processed = processed;
		}

		public String getPromoId() {
			return promoId;
		}

		public void setPromoId(String promoId) {
			this.promoId = promoId;
		}
	}
	
	public static class Bundler{
		private int id = 1;

		public int getId() {
			return id;
		}

		public void setId(int id) {
			this.id = id;
		}
		
		public void increment(){
			id++;
		}
	}
	
	

}
